package cz.frantovo.rozsireneAtributy;

import java.io.File;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.file.Path;
import java.nio.file.attribute.UserDefinedFileAttributeView;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

/**
 *
 * @author fiki
 */
public class Model implements TableModel {

	private static final Logger log = Logger.getLogger(Model.class.getSimpleName());
	private String[] sloupečky = {"Název", "Hodnota"};
	private HashSet<TableModelListener> posluchače = new HashSet<TableModelListener>();
	private UserDefinedFileAttributeView souborovySystem;
	private ArrayList<Atribut> atributy = new ArrayList<Atribut>();

	public Model(File soubor) throws IOException {
		Path cesta = soubor.toPath();
		souborovySystem = cesta.getFileAttributeView(UserDefinedFileAttributeView.class);
		načtiAtributy();
	}

	public int getRowCount() {
		return atributy.size();
	}

	public int getColumnCount() {
		return sloupečky.length;
	}

	public String getColumnName(int n) {
		return sloupečky[n];
	}

	public Class<?> getColumnClass(int n) {
		return String.class;
	}

	public boolean isCellEditable(int m, int n) {
		return true;
	}

	public Object getValueAt(int m, int n) {
		if (n == 0) {
			return atributy.get(m).getKlic();
		} else if (n == 1) {
			return atributy.get(m).getHodnota();
		} else {
			return null;
		}
	}

	public void setValueAt(Object value, int m, int n) {
		Atribut a = atributy.get(m);
		try {
			if (n == 0) {
				/** Měníme klíč – název atributu */
				String novýKlíč = String.valueOf(value.toString());
				if (!novýKlíč.equals(a.getKlic())) {
					souborovySystem.delete(a.getKlic());
					a.setKlic(novýKlíč);
					souborovySystem.write(a.getKlic(), a.getHodnotaBajty());
				}
			} else if (n == 1) {
				/** Měníme hodnotu atributu */
				a.setHodnota(String.valueOf(value.toString()));
				souborovySystem.write(a.getKlic(), a.getHodnotaBajty());
			}
		} catch (IOException e) {
			log.log(Level.SEVERE, "Selhalo ukládání atributu na souborový systém", e);
		}
	}

	public void addTableModelListener(TableModelListener l) {
		posluchače.add(l);
	}

	public void removeTableModelListener(TableModelListener l) {
		posluchače.remove(l);
	}

	public void přidejAtribut(Atribut a) {
		atributy.add(a);
		upozorniPosluchače();
	}

	public void odeberAtribut(Atribut a) {
		atributy.remove(a);
		upozorniPosluchače();
	}

	private void načtiAtributy() throws IOException {
		List<String> jménaAtributů = souborovySystem.list();
		for (String jménoAtributu : jménaAtributů) {
			ByteBuffer hodnotaAtributu = ByteBuffer.allocate(souborovySystem.size(jménoAtributu));
			souborovySystem.read(jménoAtributu, hodnotaAtributu);
			atributy.add(new Atribut(jménoAtributu, hodnotaAtributu));
		}
		upozorniPosluchače();
	}

	private void upozorniPosluchače() {
		for (TableModelListener p : posluchače) {
			p.tableChanged(new TableModelEvent(this));
		}
	}
}
